# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018-09-14
"""
from datetime import datetime
from unittest import TestCase

from ksif import Portfolio
from ksif.core.columns import *


class TestShowPlot(TestCase):

    def test_show_plot(self):
        pf = Portfolio()
        pf = pf.loc[pf[DATE] >= datetime(year=2011, month=5, day=31), :]

        pf.show_plot()

        pf.show_plot(cumulative=False, weighted=True, title='title', show_benchmark=False)




from datetime import datetime as dt
import numpy as np
# import pandas as pd
import pandas_datareader as pdr
from data._util import *
# import multiprocessing
# import os

# ---------- PRICE DATA ----------

# Get price data of the firm
def get_a_price(code, start, end, source='yahoo'):
    try:
        return pdr.DataReader(code, source, start, end)
    except:
        return False


# Get price data for Republic of KOREA, Stock market.
def price_slow(stock_list, start, end=dt.now().strftime('%Y-%m-%d'), source='yahoo'):
    # Suppose that the type of data is 'List'

    # 기업들의 기록 형식을 Yahoo finance API에 적절한 형태로 변환
    if type(stock_list) == str: stock_list = [stock_list]
    data = list(map(lambda stock: code_df['code'][list(np.where(code_df == stock)[0])[0]] + code_df['market'][
        list(np.where(code_df == stock)[0])[0]], stock_list))

    result = pd.DataFrame()
    success_list = []
    fail_list = []
    for index, stock in enumerate(data):
        data = get_a_price(stock, start, end)
        if type(data) != bool:
            success_list.append(stock)
            result = result.append(data.T)
        else:
            fail_list.append(stock)

    result.index = pd.MultiIndex.from_product([success_list, ['high', 'low', 'open', 'close', 'volume', 'adj_close']])

    return result






'''
import unittest
from pocket import Pocket
from datetime import datetime

pkt = Pocket(0)
pkt.order(datetime(2010, 9, 30), "A000030", 3)

sample_dict = {"A000660": 10, "A000400" : 100}
pkt.order(datetime(2011, 9, 30), sample_dict)

sample_dict = {"A000660": 1, "A000400": -10, "A000030" : 0}
pkt.order(datetime(2012, 9, 28), sample_dict)

class PocketTest(unittest.TestCase):
    def test(self):
        self.assertEqual(pkt.cur_stocks.at["A000660", "NAME"], "SK하이닉")

if __name__ == "__main__":
    unittest.main()
'''

# ---------- CHECK DATA PACKAGE ----------

from data import price

print(price.price_slow('삼성전자', start = '2019-01-01'))
print(price.price_slow(['삼성전자', 'SK하이닉스'], start = '2019-01-01'))

data = print(price.price_slow(['삼성전자', 'SK하이닉스'], start = '2019-01-01'))





# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 8. 24.
"""
from unittest import TestCase

from ksif import *


class TestPeriodicRank(TestCase):

    def test_periodic_rank(self):
        pf = Portfolio()
        pf = pf.periodic_rank(min_rank=26, max_rank=50)

        # Periodic sizes are 25.
        pf.groupby(by=DATE).size().apply(lambda x: self.assertEqual(x, 25))



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 9. 6.
"""
PORTFOLIO_RETURN = 'portfolio_return'
BENCHMARK_RETURN = 'benchmark_return'
ACTIVE_RETURN = 'active_return'
ACTIVE_RISK = 'active_risk'
SR = 'sharpe_ratio'
IR = 'information_ratio'
CAGR = 'compound_annual_growth_rate'
MDD = 'maximum_drawdown'
FAMA_FRENCH_ALPHA = 'Fama_French_alpha'
FAMA_FRENCH_ALPHA_P_VALUE = 'Fama_French_alpha_p_value'
FAMA_FRENCH_BETA = 'Fama_French_beta'
TURNOVER = 'turnover'



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 6. 6.
"""


class GoogleQueryException(Exception):
    message = 'Google querying is not working. The status code is {}.'

    def __init__(self, status_code: int):
        self._status_code = status_code

    def __str__(self):
        return self.message.format(self._status_code)


class EmptyResultException(Exception):
    message = "The result of the function is empty."

    def __str__(self):
        return self.message



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018-09-14
"""
from unittest import TestCase

import pandas as pd
import numpy as np
from datetime import datetime

from ksif import *


class TestOutcome(TestCase):

    def test_show_plot(self):
        pf = Portfolio()

        outcome = pf.outcome(show_plot=True)
        for value in outcome.values():
            self.assertFalse(pd.isna(value))

    def test_long_only_weighted(self):
        pf = Portfolio()

        outcome = pf.outcome(weighted=MKTCAP)
        for value in outcome.values():
            self.assertFalse(pd.isna(value))

    def test_long_short_weighted(self):
        pf = Portfolio()
        pf['weight'] = np.where(pf[PER] > 0, 1, -1)

        outcome = pf.outcome(weighted='weight')
        for value in outcome.values():
            self.assertFalse(pd.isna(value))

    def test_no_short_period(self):
        pf = Portfolio()
        pf['weight'] = np.where(pf[PER] > 0, 1, -1)
        pf.loc[pf[DATE] == datetime(2019, 1, 31), 'weight'] = 1

        outcome = pf.outcome(weighted='weight')
        for value in outcome.values():
            self.assertFalse(pd.isna(value))



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 7. 18.
"""
import time
from functools import wraps


def retry(exceptions, tries=3, delay=1, backoff=2, logger=None):
    """
    Retry calling the decorated function using an exponential backoff.

    Args:
        exceptions: The exception to check. may be a tuple of
            exceptions to check.
        tries: Number of times to try (not retry) before giving up.
        delay: Initial delay between retries in seconds.
        backoff: Backoff multiplier (e.g. value of 2 will double the delay
            each retry).
        logger: Logger to use. If None, print.
    """

    def deco_retry(func):

        @wraps(func)
        def f_retry(*args, **kwargs):
            mtries, mdelay = tries, delay
            while mtries > 1:
                try:
                    return func(*args, **kwargs)
                except exceptions as e:
                    msg = '{}, Retrying in {} seconds...'.format(e, mdelay)
                    if logger:
                        logger.warning(msg)
                    else:
                        print(msg)
                    time.sleep(mdelay)
                    mtries -= 1
                    mdelay *= backoff
            return func(*args, **kwargs)

        return f_retry

    return deco_retry



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018-09-14
"""
from unittest import TestCase

import pandas as pd

from ksif.errors import EmptyResultException
from ksif.util.checker import not_empty


class TestChecker(TestCase):

    def test_not_empty(self):
        @not_empty
        def test_empty_dataframe():
            return pd.DataFrame()

        # test_empty_dataframe should raise a EmptyResultException.
        with self.assertRaises(EmptyResultException):
            test_empty_dataframe()

        @not_empty
        def test_empty_series():
            return pd.Series()

        # test_empty_series should raise a EmptyResultException.
        with self.assertRaises(EmptyResultException):
            test_empty_series()

        @not_empty
        def test_none():
            return None

        # test_none should raise nothing.
        test_none()



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018-09-16
"""
from unittest import TestCase

from ksif import *
from ksif.core.frame import RANK_CORRELATION


class TestRankCorrelation(TestCase):

    def test_rank_correlation(self):
        pf = Portfolio()
        rank_ic = pf.rank_correlation(factor=PER, ranked_by=RET_3, rolling=12, show_plot=True,
                                      title='PER-RET_3 Rank Correlation')
        self.assertEqual(sorted(rank_ic.columns), sorted([RANK_CORRELATION, 'rolling_12']))



"""
Date : 2018. 9. 1
Author : Jiwoo Park
"""

import time
import threading
import itertools
import sys

class Spinner():
    spinner_char = itertools.cycle(['-','/','|','\\'])
    def __init__(self):
        self.stop_running = threading.Event()
        self.spin_thread = threading.Thread(target=self.init_spin)

    def start(self):
        self.spin_thread.start()

    def stop(self):
        self.stop_running.set()
        self.spin_thread.join()

    def init_spin(self):
        while not self.stop_running.is_set():
            sys.stdout.write(next(self.spinner_char))
            sys.stdout.flush()
            time.sleep(0.25)
            sys.stdout.write('\b')

def spinner(func):
    spin = Spinner()
    def inner():
        spin.start()
        func()
        spin.stop()
    return inner

@spinner
def do_work():
    time.sleep(3)

if __name__ == "__main__":
    do_work()



import pandas as pd


# ---------- CODES ----------

# Get code data easily
def code(market = 'all', attribute=['code', 'name', 'market'], listed=True):
    # print("Collecting code data for %s as requested form" % ("KOSPI & KOSDAQ" if market == 'all' else market.upper()))
    # ERROR CHECK
    if market not in ['kospi', 'kosdaq', 'all']:
        print("ERROR - Invalid value of market")
        return None

    # Get code data
    code_df = get_codeListed_df()

    # listed에 따라 데이터 추가
    if not listed:
        code_df = get_codeDelisted_df()
    elif listed == 'both':
        code_df.append(get_codeListed_df(), ignore_index = True, verify_integrity = True)

    # Eliminate unnecessary attributes
    code_df = code_df[attribute]

    # Eliminate unnecessary records
    if market != 'all':
        code_df = code_df[code_df['market'] == market]  # Specific market
        code_df = code_df.reset_index()

    return code_df

# Get listed data on KRX, stock.
def get_KOR_stockListed_df(target = ['kospi', 'kosdaq']):
    # Yahoo Finance does not support inquiring about Konnex.
    df_dict = {}

    # get_KOSPI
    kospi_code_df = \
        pd.read_html(
            'http://kind.krx.co.kr/corpgeneral/corpList.do?marketType=stockMkt&method=download&searchType=13',
            header=0)[0]
    kospi_code_df["market"] = "kospi"
    df_dict['kospi'] = kospi_code_df

    # get_KOSDAQ
    kosdaq_code_df = \
        pd.read_html(
            'http://kind.krx.co.kr/corpgeneral/corpList.do?marketType=kosdaqMkt&method=download&searchType=13',
            header=0)[0]
    kosdaq_code_df["market"] = "kosdaq"
    df_dict['kosdaq'] = kosdaq_code_df

    # get_KONNEX
    kkonex_code_df = \
        pd.read_html(
            'http://kind.krx.co.kr/corpgeneral/corpList.do?marketType=konexMkt&method=download&searchType=13',
            header=0)[0]
    kkonex_code_df["market"] = "kkonex"
    df_dict['konnex'] = kkonex_code_df

    # Merge
    target = list_caseConverter(target)
    data = []
    for mkt in target:
        data.append(df_dict[mkt])
    data = pd.concat(data, axis=0, ignore_index = True)


    # Change cloumn name to English
    data = data.rename(columns={'회사명': 'name', '종목코드': 'code', '업종': 'industry', '주요제품': 'major_products', \
                                      '상장일': 'day_of_listing', '결산월': 'settlement_month', '대표자명': 'CEO', \
                                      '홈페이지': 'home_page', '지역': 'location'})

    # convert the stock code into a proper form
    data.code = data.code.map('{:06d}'.format)

    return data


# Get listed code data
def get_codeListed_df(source = 'KR', target = 'stock'):
    # if source == '미국시장'
    # elif source == '중국시장'
    # elif source ==

    if source == 'KR':
        if target == 'stock':
            return get_KOR_stockListed_df()
        elif target == 'ETF':
            # Not yet
            pass

    return False

# Get delisted code data
def get_codeDelisted_df(source = 'KR', target = 'stock'):
    # Not yet created
    return None


# Set all elements in (List)data lower(or upper) capital.
def list_caseConverter(data, toLower = True):
    if not type(data) == list:
        print("Argument type ERROR in Kynance.util.modifyCapital_list()")
        return None

    for index, value in enumerate(data):
        if type(value) == str:
            if toLower: data[index] = value.lower()
            elif not toLower: data[index] = value.upper()

    return data


def purify(data, attribute = ['code', 'name', 'market'], convertRull = {'market' : {'kospi': '.KS', 'kosdaq': '.KQ'}}):
    # Check the validity

    # Check converted data has meaning - attribute
    for col in convertRull.keys():
        if col not in attribute:
            print("WARNING - %s is not in attribute. There's possibility of data loss." % (col))

    # Modify data by convertRull
    for col in convertRull.keys():
        for before in convertRull[col].keys():
            data.loc[data[col] == before, col] = convertRull[col][before]

    # Drop columns except in (List)attribute
    data = data[attribute]

    return data




# ---------- RUN WITH IMPORT ----------

code_df = code(market = 'all', attribute=['code', 'name', 'market'], listed=True)
yahoo_code_df = purify(code_df)
thread = 16


import pandas as pd
import numpy as np
from record import *
from copy import deepcopy

CODE_DF = pd.DataFrame(pd.read_html("./firm_code.xls")[0])
CODE_DF = CODE_DF.rename(columns=CODE_DF.iloc[0])[1:]

def code2firm(code):
    code = code[1:]
    return CODE_DF.loc[CODE_DF["종목코드"] == code]


def _pack_record(date, stocks, amount):
    if amount:
        return Record(date, [(stocks, code2firm(stocks)["회사명"], amount)])
    else:
        return Record(date, [(stock, code2firm(stock)["회사명"], amount) for stock, amount in stocks.items()])

def historic_stocks(pocket):
    historic_stocks = []
    for record in pocket.history:
        historic_stocks.extend([stock_code for stock_code, _, _ in record.assets])

    historic_stocks = list(set(historic_stocks))
    return historic_stocks

def update_row(historic_df, record):
    if not historic_df.index.empty:
        latest = deepcopy(pd.Series(data=historic_df.iloc[-1].values, index=historic_df.columns))
        for stock_code, _, amount in record.assets:
            latest[stock_code] += amount

        return latest.rename(None)

    else:
        result = pd.Series(index = historic_df.columns)
        for stock_code, _, amount in record.assets:
            result[stock_code] = amount
        return result.fillna(0)








# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 7. 18.
"""
from io import StringIO

import pandas as pd
from pandas import DataFrame
import requests
import urllib3

from ..errors import GoogleQueryException
from ..core.columns import DATE

urllib3.disable_warnings()  # Ignore InsecureRequestWarning.

SPREADSHEET_DOWNLOAD_REQUEST_URL = 'https://docs.google.com/spreadsheet/ccc?key=1e_7FLT22f0MPfk0zuisbCukWVoCKT3Rm9tPi6R7xntc&output=csv&gid={}'

CSV_FILE_DOWNLOAD_REQUEST_URL = 'https://drive.google.com/uc?export=download'

CSV_FILES_GID = '0'
HOLDING_COMPANY_GID = '1815448628'
DELISTED_COMPANY_GID = '324275710'
MERGED_COMPANY_GID = '1896384832'
RELISTED_COMPANY_GID = '1860868662'


def query_google_spreadsheet(gid) -> DataFrame:
    """
    :param gid: (str or None) 다운받을 시트의 이름.

    :return spreadsheet: (DataFrame)
    """
    if gid is not None and not isinstance(gid, str):
        raise TypeError("Parameter gid should be a None or a string.")

    spreadsheet_download_request_url = SPREADSHEET_DOWNLOAD_REQUEST_URL.format(gid)
    response = requests.get(spreadsheet_download_request_url)

    if response.status_code != 200:
        raise GoogleQueryException(response.status_code)

    spreadsheet = pd.read_csv(StringIO(response.content.decode(encoding='UTF-8', errors='strict')))

    return spreadsheet


def query_google_csv_file(csv_file_id) -> DataFrame:
    """
    :param csv_file_id: (str) 공유가능한 링크에 명시된 ID.
        예를 들어 공유가능한 링크가 https://drive.google.com/open?id=1ee9ZtqUgPAibE7YtN06e1rW5UnhEabjL 일 때,
        1ee9ZtqUgPAibE7YtN06e1rW5UnhEabjL 가 ID.

    :return csv_file: (DataFrame)
    """
    session = requests.Session()

    response = session.get(CSV_FILE_DOWNLOAD_REQUEST_URL, params={'id': csv_file_id}, stream=True)
    token = get_confirm_token(response)

    if token:
        params = {'id': csv_file_id, 'confirm': token}
        response = session.get(CSV_FILE_DOWNLOAD_REQUEST_URL, params=params, stream=True)

    csv_file = pd.read_csv(StringIO(response.content.decode(encoding='UTF-8', errors='strict')), parse_dates=[DATE])
    session.close()
    return csv_file


def get_confirm_token(response):
    for key, value in response.cookies.items():
        if key.startswith('download_warning'):
            return value

    return None



#-*- coding: utf-8 -*-
"""
Author  : Jiwoo Park
Date    : 2019. 1. 21
Desc    : Record of transaction
"""
from datetime import datetime

class Record():
    """
    Single Transaction Record
    Attributes 1. date of transaction
               2. amounts of stocks
    Methods    1. evaluation
    """
    def __init__(self, date, assets):
        """
        :param date:    Date that quant record
        :param assets:  Asset and it amount that quant wants to buy
        """
        self.date = date
        self.assets = assets


    def __str__(self):
        repr_str = "\n" \
                   "{:>15} | Amount\n" \
                   "=========================\n"\

        for code, name, amount in self.assets:
            name = (name[:4] + '..') if len(name) > 4 else name
            repr_str += "{:>8}({:<6})| {:>6}\n".format(code, name, amount)

        return repr_str.format(self.date.strftime("%Y-%m-%d"), self.assets, 3)


    def evaluate(self):
        self.purchase_unit_price = None
        self.current_price = None

if __name__ == "__main__":
    sample_asset = [("A000030", "갑돌이", 3), ("A000050", "asdfdsdf갑을병정sdfsdfsdf컴퍼니", 500)]
    rcd = Record(datetime(2019, 1, 3), sample_asset)
    print(rcd)




# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 8. 24.
"""
from unittest import TestCase

import numpy as np

from ksif import *


class TestPeriodicPercentage(TestCase):

    def test_periodic_percentage(self):
        pf = Portfolio()
        pf = pf.loc[~np.isnan(pf[PER]), :]

        selected_companies = pf.periodic_percentage(min_percentage=0.0, max_percentage=0.5, factor=PER)

        # The number of selected companies times 4 is almost equal to all companies.
        self.assertAlmostEqual(len(selected_companies) / len(pf), 0.5, delta=0.001)

        # The average PER of selected companies is bigger than the average PER of all companies.
        self.assertTrue(np.average(selected_companies[PER]) > np.average(pf[PER]))



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018-09-14
"""
from datetime import datetime
from unittest import TestCase

import numpy as np
import pandas as pd

from ksif import *


class TestOutcome(TestCase):

    def test_long_only_weighted(self):
        pf = Portfolio()

        returns = pf.get_returns()
        self.assertFalse(pd.isna(all(returns[PORTFOLIO_RETURN])))

    def test_cumulative(self):
        pf = Portfolio()

        returns = pf.get_returns(cumulative=True)
        self.assertFalse(pd.isna(all(returns[PORTFOLIO_RETURN])))

    def test_long_short_weighted(self):
        pf = Portfolio()
        pf['weight'] = np.where(pf[PER] > 0, 1, -1)

        returns = pf.get_returns(weighted='weight')
        self.assertFalse(pd.isna(all(returns[PORTFOLIO_RETURN])))

    def test_no_short_period(self):
        pf = Portfolio()
        pf['weight'] = np.where(pf[PER] > 0, 1, -1)
        pf.loc[pf[DATE] == datetime(2019, 1, 31), 'weight'] = 1

        returns = pf.get_returns(weighted='weight')
        self.assertFalse(pd.isna(all(returns[PORTFOLIO_RETURN])))



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 7. 18.
"""
import os
from pathlib import Path

import urllib3
import pandas as pd

from pandas import DataFrame
from pandas import read_hdf

from .google_drive import query_google_spreadsheet, query_google_csv_file, GoogleQueryException, CSV_FILES_GID
from ..core.columns import DATE
from ..util.memoization import memoize
from ..util.retrial import retry

urllib3.disable_warnings()  # Ignore InsecureRequestWarning.

TABLE = 'table'

COMPANY = 'company'
BENCHMARK = 'benchmark'
FACTOR = 'factor'

# CSV encoding type
ENCODING = 'utf-8'

DATA_DIR = 'data'


@memoize(copy=True)
@retry(GoogleQueryException)
def download_latest_data(download_company_data) -> (DataFrame, DataFrame):
    csv_files_url = query_google_spreadsheet(CSV_FILES_GID)
    csv_files_url = csv_files_url.sort_values(by=DATE)
    latest_company_id = csv_files_url.iloc[-1][COMPANY]
    latest_company_file_name = csv_files_url.iloc[-1][DATE] + '_' + COMPANY
    # latest_benchmark_id = csv_files_url.iloc[-1][BENCHMARK]
    # latest_benchmark_file_name = csv_files_url.iloc[-1][DATE] + '_' + BENCHMARK
    # latest_factor_id = csv_files_url.iloc[-1][FACTOR]
    # latest_factor_file_name = csv_files_url.iloc[-1][DATE] + '_' + FACTOR

    if not Path(DATA_DIR).exists():
        os.makedirs(DATA_DIR)

    if download_company_data:
        latest_company_data = _download_data(latest_company_file_name, latest_company_id)
    else:
        latest_company_data = None
    # latest_benchmark_data = _download_data(latest_benchmark_file_name, latest_benchmark_id)
    # latest_factor_data = _download_data(latest_factor_file_name, latest_factor_id)
    # latest_factor_data.set_index(DATE, inplace=True)

    # return latest_company_data, latest_benchmark_data, latest_factor_data
    return latest_company_data


def _download_data(file_name, id):
    local_company_file_path = '{}/{}.pck'.format(DATA_DIR, file_name)
    if Path(local_company_file_path).exists():
        # latest_company_data = custom_read_hdf(local_company_file_path)
        latest_company_data = pd.read_pickle(local_company_file_path)
    else:
        print("Downloading {} from web...".format(file_name))
        latest_company_data = query_google_csv_file(id)
        # latest_company_data.to_hdf(local_company_file_path, key=TABLE, mode='w')

        latest_company_data['mdate'] = pd.to_datetime(latest_company_data['mdate'])
        latest_company_data['date'] = pd.to_datetime(latest_company_data['date'])

        latest_company_data.to_pickle(local_company_file_path)
        print("{} is saved as {}.".format(file_name, local_company_file_path))
    return latest_company_data


def custom_read_hdf(path):
    latest_korea_data = read_hdf(path, key=TABLE)
    return latest_korea_data



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 8. 24.
"""
from unittest import TestCase

from pandas.util.testing import assert_frame_equal

from ksif import *


class TestBenchmark(TestCase):

    def test_benchmark(self):
        pf = Portfolio()

        # The return of Portfolio.benchmark should be str.
        self.assertEqual(type(pf.benchmark), str)

    def test_get_benchmark(self):
        pf = Portfolio()

        # The default benchmark is KOSPI.
        benchmark = pf.get_benchmark()
        self.assertEqual(benchmark.columns.tolist(),
                         [BENCHMARK_RET_1, BENCHMARK_RET_3, BENCHMARK_RET_6, BENCHMARK_RET_12])

        # If the benchmark is set, return the benchmark.
        benchmark = pf.get_benchmark(benchmark=KOSDAQ)
        self.assertEqual(benchmark.columns.tolist(),
                         [BENCHMARK_RET_1, BENCHMARK_RET_3, BENCHMARK_RET_6, BENCHMARK_RET_12])

        # If the benchmark is not exist, raise ValueError.
        with self.assertRaises(ValueError):
            pf.get_benchmark(benchmark='There is no such a benchmark.')

    def test_set_benchmark(self):
        pf = Portfolio()

        # If the benchmark is not exist, raise ValueError.
        with self.assertRaises(ValueError):
            pf.set_benchmark(benchmark='There is no such a benchmark.')

        # If the benchmark is exist, change the default benchmark.
        kosdaq_1 = pf.get_benchmark(benchmark=KOSDAQ)
        pf.set_benchmark(benchmark=KOSDAQ)
        kosdaq_2 = pf.get_benchmark()

        assert_frame_equal(kosdaq_1, kosdaq_2)



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018-09-16
"""
from datetime import datetime
from unittest import TestCase

from ksif import *


class TestQuantileDistributionRatio(TestCase):

    def test_quantile_distribution_ratio(self):
        pf = Portfolio(start_date=datetime(year=2017, month=1, day=1))

        # When chunk_num is less than or equal to 1, raise an assertion error.
        with self.assertRaises(AssertionError):
            pf.quantile_distribution_ratio(factor=MKTCAP, chunk_num=1)
        with self.assertRaises(AssertionError):
            pf.quantile_distribution_ratio(factor=MKTCAP, chunk_num=-1)

        # All true parameters with a title.
        # The length of the result data frame is equal to original unique DATE minus one.
        date_num = len(pf[DATE].unique())
        result_data = pf.quantile_distribution_ratio(factor=MKTCAP, cumulative=True, weighted=True, only_positive=True,
                                                     show_plot=True, show_bar_chart=True, title='title')
        self.assertEqual(date_num - 1, len(result_data))

        # All true parameters with a title.
        # The length of the result data frame is equal to original unique DATE minus one.
        date_num = len(pf[DATE].unique())
        result_data = pf.quantile_distribution_ratio(factor=MKTCAP, cumulative=False, weighted=False,
                                                     only_positive=False, show_plot=False, show_bar_chart=False)
        self.assertEqual(date_num - 1, len(result_data))



"""
Author  : Jiwoo Park
Date    : 2019. 1. 21
Desc    : Pocket - Balance Record System for Quant
"""
import sys
import pandas as pd
import numpy as np

from record import Record
from datetime import datetime
import util

PRICE_DF = pd.read_csv("./PRICE.csv", index_col=[0], parse_dates=[0])


class Pocket():
    """
    Balance System
    """
    def __init__(self, cash):
        """
        :param cash:    (Int) Amount of Cash in the pocket
        :param cur_stocks:  (Dataframe) of Stocks and its amount in the pocket
                            Which Column has [code, name, amount, purchase_date, purchase_price,
                                             purchase_volume, current_price, current_volume, return]
        """
        self.cash = cash
        columns = ["NAME", "AMOUNT", "DATE_PURCHASE", "PRICE_PURCHASE", "VOLUME_PURCHASE"]
        self.cur_stocks = pd.DataFrame(columns=columns) # cur_stock : code, name, amount, price_purchase, volume_purchase, return,
                                                        # price_current
        self.history = []
        self.logs = False


    def _set_one_stock(self, stock, amount, date):
        # buy or sell unit stock
        target_firm = util.code2firm(stock)
        try:
            price_at_date = PRICE_DF.loc[date]
        except Exception as e:
            print(e)
            raise ValueError("Invalid Date to Order - by KSIF Tech")

        try:
            price_of_firm = price_at_date[stock]
            if not price_of_firm:
                raise ValueError("Ordered Stock doesn't have proper price info.")
        except Exception as e:
            print(e)
            raise ValueError("Ordered Stock is not available to trade. - by KSIF Tech")

        # if stock already in position, how to do it??? --> Where to put cash concept in abstraction.
        if stock not in self.cur_stocks.index:
            self.cur_stocks.loc[stock] = [target_firm["회사명"].values[0], amount, date,
                                          price_of_firm, price_of_firm * amount]
        elif amount == 0:
            self.cur_stocks = self.cur_stocks.drop(stock)

        else:
            self.cur_stocks.at[stock, "AMOUNT"] += amount
            self.cur_stocks.at[stock, "VOLUME_PURCHASE"] += price_of_firm * amount

    def _set_cur_stocks(self, stocks, amount, date):
        if isinstance(stocks, type("")):
            self._set_one_stock(stocks, amount, date)
        elif isinstance(stocks, type({})):
            for stock, amount in stocks.items():
                self._set_one_stock(stock, amount, date)

    def order(self, date, stocks, amount=None):
        # Set cur_stocks and Add new record to history
        success = True
        if not date:
            raise ValueError("Date is Required")

        try:
            new_record = util._pack_record(date, stocks, amount)
            self._set_cur_stocks(stocks, amount, date)
            self.history.append(new_record)
        except Exception as e:
            print("Your Order has been failed because of following reason.")
            success = False
            print(e)

        if self.logs and success:
            print(new_record)

        return success

    def view_history(self):
        for hist in self.history:
            print(hist)

    def evaluate_cur_stocks(self):
        """
        Evaluate current holding stocks at Today and update cur stock attribute
        :return: No return
        """
        today = datetime.today()
        close_val = PRICE_DF.iloc[PRICE_DF.index.get_loc(today, method="ffill")]
        close_val = close_val[self.cur_stocks.index]
        close_val = pd.DataFrame({"PRICE_CURRENT" : close_val.values}, index=self.cur_stocks.index)
        evaluated_stocks = pd.merge(self.cur_stocks, close_val, left_index=True, right_index=True)
        evaluated_stocks["VOLUME_CURRENT"] = evaluated_stocks["AMOUNT"] * evaluated_stocks["PRICE_CURRENT"]
        evaluated_stocks["RETURN"] = (evaluated_stocks["VOLUME_CURRENT"] / evaluated_stocks["VOLUME_PURCHASE"]) - 1
        return evaluated_stocks

    def evaluate_history(self, price_info):
        """
        Make historical change of price for pocket
        :return: Historical DataFrame for holding stocks
        """

        historic_stocks = util.historic_stocks(self)
        historic_df = pd.DataFrame(columns=historic_stocks)

        for record in self.history:
            update_row = util.update_row(historic_df, record)
            historic_df.loc[record.date] = update_row
            print(historic_df)

        start_date = self.history[0].date
        end_date = self.history[-1].date

        price_info = price_info.loc[(price_info.index >= start_date) & (price_info.index <= end_date)][historic_stocks]
        historic_stocks = price_info.merge(historic_df,
                                           how="left", left_index=True, right_index=True,
                                           suffixes=("_price", "_amount"))
        return historic_stocks


    def flush(self):
        """ Make Pocket cur holding stock to empty
        :return:
        """
        self.cur_stocks = self.cur_stocks.drop(self.cur_stocks.index)
        # add history that flushed whole stocks

    def cal_return(self, price_info):
        # pick trades belongs to start ~ end
        # for those trades calculate returns each

        self.evaluate_history(price_info)


    def analyze(self, start, end):
        """ Analyzing the performance of manager's pocket.
        :param start:   (datetime) Datetime starting analyzing date
        :param end:     (datetime) Datetime ending analyzing date
        :return:        (DataFrame) Dataframe that contain summary statistics
        """
        return



def get_update_list(new_portfolio_dict, portfolio_dict):
    enterance_dict = {}
    for stock_code, quantity in new_portfolio_dict.items():
        if stock_code not in portfolio_dict:
            enterance_dict[stock_code] = quantity

    exit_dict = {}
    for stock_code, quantity in portfolio_dict.items():
        if stock_code not in new_portfolio_dict:
            exit_dict[stock_code] = quantity

    update_stock_list = []

    update_portfolio_list = []


if __name__ == "__main__":
    pkt = Pocket(0)
    pkt.logs = False # Activate Logging
    PRICE_DF = pd.read_csv("./PRICE.csv", index_col=[0], parse_dates=[0])

    pkt.order(datetime(2010, 9, 30), "A000030", 3)

    sample_dict = {"A000660": 10, "A000400" : 100}
    pkt.order(datetime(2011, 9, 30), sample_dict)

    sample_dict = {"A000660": 1, "A000400": -10, "A000030" : 0}
    pkt.order(datetime(2012, 9, 28), sample_dict)
    # pkt.cal_return(datetime(2011, 5, 8), datetime(2013, 10, 20))
    pkt.cal_return(PRICE_DF)




# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
         Park Ji woo
:Date: 2018. 7. 18
"""
import os
import platform
import sys
from copy import deepcopy as dc
from datetime import datetime
from warnings import warn

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import pandas.core.common as com
import statsmodels.api as sm
from matplotlib import font_manager, rc
from pandas import DataFrame
from pandas import Series
from pandas.core.index import MultiIndex
from pandas.core.indexing import convert_to_index_sliceable
from performanceanalytics.charts.performance_summary import create_performance_summary

from .columns import *
from .outcomes import *
from ..io.downloader import download_latest_data
from ..util.checker import not_empty
import io

# Hangul font setting
# noinspection PyProtectedMember
font_manager._rebuild()
if platform.system() == 'Windows':
    font_name = font_manager.FontProperties(fname='c:/Windows/Fonts/malgun.ttf').get_name()
elif platform.system() == 'Darwin':  # OS X
    font_name = font_manager.FontProperties(fname='/Library/Fonts/AppleGothic.ttf').get_name()
else:  # Linux
    fname = '/usr/share/fonts/truetype/nanum/NanumGothicOTF.ttf'
    if not os.path.isfile(fname):
        raise ResourceWarning("Please install NanumGothicOTF.ttf for plotting Hangul.")
    font_name = font_manager.FontProperties(fname=fname).get_name()
rc('font', family=font_name)

# for fix broken Minus sign
matplotlib.rcParams['axes.unicode_minus'] = False

PERCENTAGE = 'percentage'

WEIGHT = 'weight'
WEIGHT_SUM = 'weight_sum'

START_DATE = datetime(year=2001, month=5, day=31)

QUANTILE = 'quantile'
RANK = 'rank'
RANK_CORRELATION = 'Rank correlation'


class Portfolio(DataFrame):
    """

    """
    _benchmark = KOSPI
    benchmarks = None
    factors = None

    @property
    def _constructor(self):
        return Portfolio

    @not_empty
    def __init__(self, data=None, index=None, columns=None, dtype=None, copy: bool = False,
                 start_date: str = START_DATE, end_date: str = None,
                 include_holding: bool = False, include_finance: bool = False,
                 include_managed: bool = False, include_suspended: bool = False):

        # try:
        #     datetime.strptime(start_date, '%Y-%m-%d')
        # except ValueError:
        #     raise ValueError("Incorrect data format, start_date should be YYYY-MM-DD")

        # if not end_date:
        #     end_date = datetime.today().strftime('%Y-%m-%d')

        # try:
        #     datetime.strptime(end_date, '%Y-%m-%d')
        # except ValueError:
        #     raise ValueError("Incorrect data format, end_date should be YYYY-MM-DD")

        if data is None:
            # data, self.benchmarks = download_latest_data()
            download_company_data = True
            data = download_latest_data(download_company_data)

            # if not include_holding:
            #     data = data.loc[~data[HOLDING], :]
            #
            # if not include_finance:
            #     data = data.loc[data[FN_GUIDE_SECTOR] != '금융', :]
            #
            # if not include_managed:
            #     data = data.loc[~data[IS_MANAGED], :]
            #
            # if not include_suspended:
            #     data = data.loc[~data[IS_SUSPENDED], :]
            #
            # data = data.loc[(start_date <= data[DATE]) & (data[DATE] <= end_date), :]
        else:
            download_company_data = True
            # _, self.benchmarks = download_latest_data()
            _ = download_latest_data(download_company_data)

        # DataFrame.__init__(self=self, data=data, index=index, columns=columns, dtype=dtype, copy=copy)

        super(Portfolio, self).__init__(data=data)

    # def __getitem__(self, key):
    #     from pandas.core.dtypes.common import is_list_like, is_integer, is_iterator
    #
    #     key = com.apply_if_callable(key, self)
    #
    #     # shortcut if the key is in columns
    #     try:
    #         if self.columns.is_unique and key in self.columns:
    #             if self.columns.nlevels > 1:
    #                 return self._getitem_multilevel(key)
    #             return self._get_item_cache(key)
    #     except (TypeError, ValueError):
    #         # The TypeError correctly catches non hashable "key" (e.g. list)
    #         # The ValueError can be removed once GH #21729 is fixed
    #         pass
    #
    #     # Do we have a slicer (on rows)?
    #     indexer = convert_to_index_sliceable(self, key)
    #     if indexer is not None:
    #         return self._slice(indexer, axis=0)
    #
    #     # Do we have a (boolean) DataFrame?
    #     if isinstance(key, DataFrame):
    #         return self._getitem_frame(key)
    #
    #     # Do we have a (boolean) 1d indexer?
    #     if com.is_bool_indexer(key):
    #         return self._getitem_bool_array(key)
    #
    #     # We are left with two options: a single key, and a collection of keys,
    #     # We interpret tuples as collections only for non-MultiIndex
    #     is_single_key = isinstance(key, tuple) or not is_list_like(key)
    #
    #     if is_single_key:
    #         if self.columns.nlevels > 1:
    #             return self._getitem_multilevel(key)
    #         indexer = self.columns.get_loc(key)
    #         if is_integer(indexer):
    #             indexer = [indexer]
    #     else:
    #         if is_iterator(key):
    #             key = list(key)
    #         # noinspection PyProtectedMember
    #         indexer = self.loc._convert_to_indexer(key, axis=1, raise_missing=True)
    #
    #     # take() does not accept boolean indexers
    #     if getattr(indexer, "dtype", None) == bool:
    #         indexer = np.where(indexer)[0]
    #
    #     data = self._take(indexer, axis=1)
    #
    #     if is_single_key:
    #         # What does looking for a single key in a non-unique index return?
    #         # The behavior is inconsistent. It returns a Series, except when
    #         # - the key itself is repeated (test on data.shape, #9519), or
    #         # - we have a MultiIndex on columns (test on self.columns, #21309)
    #         if data.shape[1] == 1 and not isinstance(self.columns, MultiIndex):
    #             data = data[key]
    #
    #     return data

    @property
    def benchmark(self):
        return self._benchmark

    @not_empty
    def get_benchmark(self, benchmark: str = None) -> DataFrame:
        """
        Return a benchmark of this portfolio period.

        :param benchmark: (str) The name of benchmark to use. If benchmark is None, use self.benchmark.

        :return selected_benchmark: (DataFrame)
            code                | (str) The name of benchmark. ex) 코스피, 코스닥...
            date                | (datetime)
            benchmark_return_1  | (float) 1 month return.
            benchmark_return_3  | (float) 3 month return.
            benchmark_return_6  | (float) 6 month return.
            benchmark_return_12 | (float) 12 month return.
        """
        if benchmark is not None and benchmark not in BENCHMARKS:
            raise ValueError('{} is not registered.'.format(benchmark))

        if benchmark:
            selected_benchmark = self.benchmarks.loc[(self.benchmarks[CODE] == benchmark) &
                                                     (self.benchmarks[DATE] >= min(self[DATE])) &
                                                     (self.benchmarks[DATE] <= max(self[DATE])), :]
        else:
            selected_benchmark = self.benchmarks.loc[(self.benchmarks[CODE] == self._benchmark) &
                                                     (self.benchmarks[DATE] >= min(self[DATE])) &
                                                     (self.benchmarks[DATE] <= max(self[DATE])), :]

        selected_benchmark.set_index(DATE, inplace=True)
        selected_benchmark = selected_benchmark.loc[
                             :, [BENCHMARK_RET_1, BENCHMARK_RET_3, BENCHMARK_RET_6, BENCHMARK_RET_12]
                             ]
        return selected_benchmark

    def set_benchmark(self, benchmark):
        if benchmark not in BENCHMARKS:
            raise ValueError('{} is not registered.'.format(benchmark))
        else:
            self._benchmark = benchmark

    # noinspection PyPep8Naming
    @property
    def SMB(self) -> Series:
        return self.factors.loc[:, SMB]

    # noinspection PyPep8Naming
    @property
    def HML(self) -> Series:
        return self.factors.loc[:, HML]

    @not_empty
    def to_dataframe(self, deepcopy: bool = True) -> DataFrame:
        """
        Convert portfolio to dataframe type.

        :param deepcopy : (bool) If deepcopy is True, convert to dataframe based on deepcopy. Or, convert to dataframe
                                  based on shallow copy.

        :return dataframe : (DataFrame) Converted dataframe type portfolio
        """
        if deepcopy:
            dataframe = DataFrame(dc(self))
        else:
            dataframe = DataFrame(self)

        return dataframe

    def outcome(self, benchmark: str = None, weighted: str = None,
                long_transaction_cost_ratio: float = 0.0025, short_transaction_cost_ratio: float = 0.0025,
                show_plot: bool = False):
        """
        Calculate various indices of the portfolio.

        :param benchmark: (str) The name of benchmark. If benchmark is None, use a default benchmark.
        :param weighted: (str) If weighted is a string, use the string to calculate weighted portfolio.
                                If there are negative weights, calculate long-short weighted portfolio.
        :param long_transaction_cost_ratio: (float) A transaction cost ratio for long investment
        :param short_transaction_cost_ratio: (float) A transaction cost ratio for short investment
        :param show_plot: (bool) If show_plot is True, show a performance summary graph.

        :return result: (dict)
            portfolio_return            | (float) Total compound return of the portfolio
            benchmark_return            | (float) Total compound return of the benchmark
            active_return               | (float) Annualized average excess return
            active_risk                 | (float) Annualized tracking error
            sharpe_ratio                | (float) Sharpe ratio
            information_ratio           | (float) Average excess return / tracking error
            compound_annual_growth_rate | (float) Annual compound return of the portfolio
            maximum_drawdown            | (float) The maximum loss from a peak to a trough of a portfolio,
                                                  before a new peak is attained
            Fama_French_alpha           | (float) An abnormal return from Fama-French 3 Factor model
                                                  (Fama and French, 1993)
            Fama_French_alpha_p_value   | (float) A p-value of the the abnormal return
            Fama_French_beta            | (float) A market beta from Fama-French 3 Factor model
                                                  (Fama and French, 1993)
            turnover                    | (float) Annual average turnover
        """
        if benchmark is not None and benchmark not in BENCHMARKS:
            raise ValueError('{} is not registered.'.format(benchmark))

        if benchmark is None:
            benchmark = self._benchmark

        portfolio, portfolio_returns, turnovers = self.get_returns_and_turnovers(long_transaction_cost_ratio,
                                                                                 short_transaction_cost_ratio, weighted)

        turnover = turnovers.mean() * 12

        benchmarks = portfolio.get_benchmark(benchmark=benchmark).loc[:, [BENCHMARK_RET_1]]
        merged_returns = pd.merge(portfolio_returns, benchmarks, on=DATE)
        merged_returns = pd.merge(merged_returns,
                                  portfolio.get_benchmark(CD91).rename(columns={BENCHMARK_RET_1: CD91}).loc[:, [CD91]],
                                  on=DATE)

        # Portfolio return, benchmark return
        portfolio_return = self._calculate_total_return(merged_returns[PORTFOLIO_RETURN])
        benchmark_return = self._calculate_total_return(merged_returns[BENCHMARK_RET_1])

        # CAGR
        period_len = len(portfolio[DATE].unique())
        compound_annual_growth_rate = (portfolio_return + 1) ** (12 / period_len) - 1

        # Active return, active risk, information ratio
        benchmark_excess_returns = merged_returns[PORTFOLIO_RETURN] - merged_returns[BENCHMARK_RET_1]

        average_excess_return = np.average(benchmark_excess_returns)
        tracking_error = np.std(benchmark_excess_returns)

        active_return = average_excess_return * 12
        active_risk = tracking_error * np.sqrt(12)
        information_ratio = average_excess_return / tracking_error

        # Sharpe ratio
        risk_free_excess_returns = merged_returns[PORTFOLIO_RETURN] - merged_returns[CD91]
        sharpe_ratio = np.average(risk_free_excess_returns) / np.std(risk_free_excess_returns)

        # Maximum drawdown
        portfolio_cumulative_assets = merged_returns[PORTFOLIO_RETURN].add(1).cumprod()
        maximum_drawdown = portfolio_cumulative_assets.div(portfolio_cumulative_assets.cummax()).sub(1).min()

        # Fama-French, 1993
        market_excess_returns = merged_returns[BENCHMARK_RET_1] - merged_returns[CD91]
        risk_free_excess_return = 'risk_free_excess_return'
        market_excess_return = 'market_excess_return'
        ff_data = pd.concat([
            DataFrame(risk_free_excess_returns, columns=[risk_free_excess_return]),
            DataFrame(market_excess_returns, columns=[market_excess_return]),
            self.factors
        ], axis=1, join='inner').dropna()
        model = sm.OLS(
            ff_data.loc[:, risk_free_excess_return],
            sm.add_constant(ff_data.loc[:, [market_excess_return, SMB, HML]])
        ).fit()
        fama_french_alpha = model.params[0]
        fama_french_alpha_p_value = model.pvalues[0]
        fama_french_beta = model.params[1]

        result = {
            PORTFOLIO_RETURN: portfolio_return,
            BENCHMARK_RETURN: benchmark_return,
            ACTIVE_RETURN: active_return,
            ACTIVE_RISK: active_risk,
            SR: sharpe_ratio,
            IR: information_ratio,
            CAGR: compound_annual_growth_rate,
            MDD: maximum_drawdown,
            FAMA_FRENCH_ALPHA: fama_french_alpha,
            FAMA_FRENCH_ALPHA_P_VALUE: fama_french_alpha_p_value,
            FAMA_FRENCH_BETA: fama_french_beta,
            TURNOVER: turnover,
        }

        if show_plot:
            plotting_returns = dc(merged_returns).loc[:, [PORTFOLIO_RETURN, BENCHMARK_RET_1]]
            plotting_returns.rename(columns={
                PORTFOLIO_RETURN: 'Portfolio',
                BENCHMARK_RET_1: benchmark
            }, inplace=True)
            create_performance_summary(plotting_returns, other_cols=range(1, 2))
            plt.show()

        return result

    def get_returns(self, weighted: str = None,
                    long_transaction_cost_ratio: float = 0.0025,
                    short_transaction_cost_ratio: float = 0.0025, cumulative=False) -> DataFrame:
        _, returns, _ = self.get_returns_and_turnovers(
            long_transaction_cost_ratio, short_transaction_cost_ratio, weighted
        )
        if cumulative:
            returns = _cumulate(returns)
        return returns

    def get_returns_and_turnovers(self, long_transaction_cost_ratio, short_transaction_cost_ratio, weighted):
        portfolio = self.dropna(subset=[RET_1])
        returns = pd.DataFrame()
        if weighted:
            if weighted not in self.columns:
                raise ValueError('{} is not in Portfolio.columns.'.format(weighted))
            portfolio = portfolio.dropna(subset=[weighted])
            long_portfolio = portfolio.loc[portfolio[weighted] > 0, :]
            short_portfolio = portfolio.loc[portfolio[weighted] < 0, :]
            short_portfolio.loc[:, RET_1] = -1 * short_portfolio.loc[:, RET_1]
            short_portfolio.loc[:, weighted] = -short_portfolio.loc[:, weighted]
            long_returns = long_portfolio.groupby([DATE]).apply(
                lambda x: np.average(x[RET_1], weights=x[weighted])
            )
            short_returns = short_portfolio.groupby([DATE]).apply(
                lambda x: np.average(x[RET_1], weights=x[weighted])
            )
            long_turnovers = _get_turnovers(long_portfolio, weighted)
            short_turnovers = _get_turnovers(short_portfolio, weighted)
            if short_returns.empty:
                returns[PORTFOLIO_RETURN] = long_returns.subtract(
                    long_turnovers.multiply(long_transaction_cost_ratio), fill_value=0)
                turnovers = long_turnovers
            else:
                returns[PORTFOLIO_RETURN] = long_returns.subtract(
                    long_turnovers.multiply(long_transaction_cost_ratio), fill_value=0
                ).add(
                    short_returns.subtract(
                        short_turnovers.multiply(short_transaction_cost_ratio), fill_value=0)
                )
                if pd.isna(returns[PORTFOLIO_RETURN]).any():
                    warn("When calculating long-short portfolio, weighted should have positive and negative values "
                         "in same periods. Otherwise, the return of the period is not calculated.")
                    returns.dropna(inplace=True)
                turnovers = long_turnovers.add(short_turnovers)
        else:
            turnovers = _get_turnovers(portfolio)
            returns[PORTFOLIO_RETURN] = portfolio.groupby([DATE]).apply(
                lambda x: np.average(x[RET_1])
            ).subtract(turnovers.multiply(short_transaction_cost_ratio), fill_value=0)
        return portfolio, returns, turnovers

    @not_empty
    def _calculate_total_return(self, grouped_data):
        data = grouped_data.dropna()
        total_return = _cumulate(data).iloc[-1]
        return total_return

    @not_empty
    def periodic_rank(self, min_rank: int = 1, max_rank: int = sys.maxsize, factor: str = MKTCAP,
                      bottom: bool = False, drop_rank: bool = True):
        """
        Select companies which have a rank bigger than or equal to min_rank, and smaller than or equal to max_rank
        for each period.

        :param min_rank: (int) The minimum rank of selected companies.
                               The selected_companies includes the minimum ranked company.
        :param max_rank: (int) The maximum rank of selected companies.
                               The selected_companies includes the maximum ranked company.
        :param factor: (str) The factor used to determine rank.
        :param bottom: (bool) If bottom is True, select the companies from bottom. Or, select the companies from top.
        :param drop_rank: (bool) If drop_rank is True, delete rank column from the selected_companies.

        :return selected_companies: (DataFrame) Selected companies for each period by rank of the factor.
        """
        assert min_rank > 0, "min_rank should be bigger than 0."
        assert max_rank > min_rank, "max_rank should be bigger than min_rank."

        all_companies = dc(self)
        all_companies = all_companies.dropna(subset=[factor])
        all_companies[RANK] = all_companies.groupby(by=[DATE])[factor].transform(
            lambda x: x.rank(ascending=bottom)
        )
        selected_companies = all_companies.loc[(all_companies[RANK] >= min_rank) & (all_companies[RANK] <= max_rank), :]
        selected_companies = selected_companies.sort_values(by=[DATE, RANK])

        if drop_rank:
            del selected_companies[RANK]

        return selected_companies

    @not_empty
    def periodic_percentage(self, min_percentage: float, max_percentage: float, factor: str = MKTCAP,
                            bottom: bool = False):
        """
        Select companies which have a percentage bigger than or equal to min_percentage, and smaller than or equal to
        max_percentage for each period.

        :param min_percentage: (float) The minimum percentage of selected companies.
                               The selected_companies includes the minimum percent company.
        :param max_percentage: (float) The maximum percentage of selected companies.
                               The selected_companies does not include the maximum percent company.
        :param factor: (str) The factor used to determine rank.
        :param bottom: (bool) If bottom is True, select the companies from bottom. Or, select the companies from top.

        :return selected_companies: (DataFrame) Selected companies for each period by quantile of the factor.
        """
        assert min_percentage >= 0, "min_percentage should be bigger than or equal to 0."
        assert max_percentage > min_percentage, "max_percentage should be bigger than min_percentage."
        assert max_percentage <= 1, "max_percentage should be smaller than or equal to 0."

        all_companies = dc(self)
        all_companies = all_companies.dropna(subset=[factor])
        all_companies[PERCENTAGE] = all_companies.groupby(by=[DATE])[factor].transform(
            lambda x: x.rank(ascending=bottom, pct=True)
        )
        selected_companies = all_companies.loc[
                             (all_companies[PERCENTAGE] >= min_percentage) &
                             (all_companies[PERCENTAGE] < max_percentage), :]

        del selected_companies[PERCENTAGE]

        return selected_companies

    @not_empty
    def periodic_standardize(self, factor: str, prefix: str = 'std_'):
        """
        Standardize a factor periodically.

        :param factor: (str) The name of factor will be standardized.
        :param prefix: (str) The prefix preceding a name of standardized factor.

        :return standardized_companies: (DataFrame) Standardized companies for each period by factor.
        """
        unstandardized_companies = dc(self.loc[~np.isnan(self[factor]), :])
        unstandardized_companies[prefix + factor] = unstandardized_companies.groupby(by=[DATE])[factor].transform(
            lambda x: (x - x.mean()) / x.std()
        )
        standardized_companies = unstandardized_companies

        return standardized_companies

    @not_empty
    def quantile_distribution_ratio(self, factor: str, chunk_num: int = 10, cumulative: bool = True,
                                    weighted: bool = False, only_positive: bool = False, show_plot: bool = False,
                                    show_bar_chart: bool = False, title: str = None) -> DataFrame:
        """
        Make quantile portfolios by the given factor, and calculate returns.

        :param factor: (str) The name of factor used to make quantile portfolios.
        :param chunk_num: (int) The number of portfolios.
        :param cumulative: (bool) If cumulative is true, calculate cumulative returns.
        :param weighted: (bool) If weighted is true, each portfolio is a weighted portfolio based on MKTCAP
        :param only_positive: (bool) If only_positive is true, use only positive value of the factor.
        :param show_plot: (bool) If show_plot is true, show a time series line chart of groups.
        :param show_bar_chart: (bool) If show_bar_chart is true, show a arithmetic average bar chart of groups.
        :param title: (str) If title is not None, set the title.

        :return quantile_portfolio_returns: (DataFrame) The returns of each group
            --------------------------------------------------------------
            date    | (datetime)
            --------------------------------------------------------------
            1       | (float) The return of group 1 portfolio at the date.
            2       | (float) The return of group 2 portfolio at the date.
            3       | (float) The return of group 3 portfolio at the date.
            ...
            --------------------------------------------------------------
        """
        assert chunk_num > 1, "chunk_num should be bigger than 1."

        labels = [str(x) for x in range(1, chunk_num + 1)]

        portfolio = dc(self)
        portfolio = portfolio.dropna(subset=[factor, RET_1])

        if only_positive:
            portfolio = portfolio.loc[portfolio[factor] > 0, :]

        portfolio[QUANTILE] = portfolio.groupby(by=[DATE])[factor].transform(
            lambda x: pd.qcut(x, chunk_num, labels=labels, duplicates='drop')
        )
        portfolio[QUANTILE] = portfolio[QUANTILE].apply(int).apply(str)

        quantile_portfolio_returns = DataFrame()
        for label in labels:
            labelled_data = portfolio.loc[portfolio[QUANTILE] == label, :]
            if weighted:
                grouped_data = labelled_data.groupby([DATE]).apply(lambda x: np.average(x[RET_1], weights=x[MKTCAP]))
            else:
                grouped_data = labelled_data.groupby([DATE])[RET_1].mean()
            grouped_data = grouped_data.rename(label)
            grouped_data = _cumulate(grouped_data, cumulative)
            quantile_portfolio_returns = pd.concat([quantile_portfolio_returns, grouped_data], axis=1, sort=True)

        if show_plot:
            plt.figure()
            quantile_portfolio_returns.plot()
            if title:
                plt.title(title)
            else:
                plt.title(factor.upper())
            plt.ylabel("Return")
            plt.xlabel("Date")
            plt.legend(loc='upper left')
            plt.show()

        if show_bar_chart:
            plt.figure()
            quantile_result = portfolio.quantile_distribution_ratio(
                factor, chunk_num=chunk_num, cumulative=False, weighted=weighted, only_positive=only_positive,
                show_plot=False, show_bar_chart=False, title=None
            )
            quantile_result.mean(axis=0).plot(kind='bar')
            if title:
                plt.title(title)
            else:
                plt.title(factor.upper())
            plt.ylabel("Return")
            plt.xlabel("Group")
            plt.show()

        return quantile_portfolio_returns

    def rank_correlation(self, factor: str, ranked_by: str = RET_1, rolling: int = 6,
                         show_plot=False, title: str = '') -> DataFrame:
        portfolio = dc(self.dropna(subset=[ranked_by]))
        portfolio = portfolio.periodic_rank(factor=factor, drop_rank=False)
        factor_rank = "{factor}_rank".format(factor=factor)
        portfolio = portfolio.rename(index=str, columns={"rank": factor_rank})
        portfolio = portfolio.periodic_rank(factor=ranked_by, drop_rank=False)
        actual_rank = "{ranked_by}_rank".format(ranked_by=ranked_by)
        portfolio = portfolio.rename(index=str, columns={"rank": actual_rank})
        rank_ic = portfolio.groupby(by=[DATE]).apply(
            lambda x: 1 - (6 * ((x[factor_rank] - x[actual_rank]) ** 2).sum()) / (len(x) * (len(x) ** 2 - 1)))

        rank_ic = pd.DataFrame(rank_ic, columns=[RANK_CORRELATION])
        rolling_column_name = 'rolling_{}'.format(rolling)
        rank_ic[rolling_column_name] = rank_ic[RANK_CORRELATION].rolling(window=rolling).mean()

        if show_plot:
            rank_ic.plot()
            plt.title(title)
            plt.axhline(y=0, color='black')
            plt.ylabel('Rank IC')
            plt.xlabel('Date')
            plt.show()

        return rank_ic

    def show_plot(self, cumulative: bool = True, weighted: bool = False, title: str = None,
                  show_benchmark: bool = True, save: bool = False):
        portfolio = self.dropna(subset=[RET_1])

        if weighted:
            grouped_data = portfolio.groupby([DATE]).apply(lambda x: np.average(x[RET_1], weights=x[MKTCAP]))
        else:
            grouped_data = portfolio.groupby([DATE])[RET_1].mean()

        # noinspection PyProtectedMember
        grouped_data = _cumulate(grouped_data, cumulative)

        plt.figure()

        if show_benchmark:
            benchmark = self.get_benchmark().loc[:, [BENCHMARK_RET_1]]
            benchmark = _cumulate(benchmark, cumulative).dropna().reset_index(drop=False)
            grouped_data = grouped_data.reset_index(drop=False)
            grouped_data = pd.merge(grouped_data, benchmark, on=[DATE])
            grouped_data = grouped_data.rename(index=str, columns={
                RET_1: 'Portfolio',
                BENCHMARK_RET_1: self.benchmark
            })
            grouped_data = grouped_data.set_index(keys=[DATE])

        grouped_data.plot()

        if title:
            plt.title(title)
        else:
            plt.title("Portfolio Simulation")
        plt.ylabel("Return")
        plt.xlabel("Date")

        # save figure
        if save:
            plt.savefig(title)

        plt.show()


def _cumulate(ret, cumulative=True):
    ret.iloc[0] = 0
    if cumulative:
        ret = ret + 1
        ret = ret.cumprod()
        ret = ret - 1
    return ret


def _get_turnovers(portfolio: Portfolio, weight: str = None) -> Portfolio:
    all_companies = Portfolio(
        include_finance=True, include_holding=True, include_managed=True, include_suspended=True,
        start_date=portfolio[DATE].min(), end_date=portfolio[DATE].max()
    )[[DATE, CODE]]

    if weight:
        selected_portfolio = portfolio[[DATE, CODE, weight]]
    else:
        selected_portfolio = portfolio[[DATE, CODE]]
        selected_portfolio.loc[:, WEIGHT] = 1
        weight = WEIGHT

    weight_sum = selected_portfolio.groupby(DATE)[weight].sum()
    weight_sum.name = WEIGHT_SUM
    selected_portfolio = selected_portfolio.merge(weight_sum, on=DATE)
    selected_portfolio[weight] = selected_portfolio[weight] / selected_portfolio[WEIGHT_SUM]
    selected_portfolio = selected_portfolio[[DATE, CODE, weight]]
    selected_portfolio = selected_portfolio.merge(
        all_companies, how='outer', on=[DATE, CODE]).fillna(0).sort_values([DATE, CODE])
    selected_portfolio[TURNOVER] = selected_portfolio.groupby(CODE)[weight].apply(
        lambda x: x - x.shift(-1)
    )
    selected_portfolio = selected_portfolio.loc[selected_portfolio[TURNOVER] >= 0, :]
    selected_portfolio[TURNOVER] = selected_portfolio[TURNOVER].fillna(selected_portfolio[weight])
    turnovers = selected_portfolio.groupby(DATE)[TURNOVER].sum()

    return turnovers



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim, Chankyu Choi, Sangheum Cho
:Date: 2018. 6. 6.
"""
from setuptools import setup, find_packages

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='ksif',
    version='v20200402_02',
    description='Quantitative investment tools for KSIF',
    long_description=long_description,
    author='KSIF Tech',
    author_email='sh.cho@kaist.ac.kr',
    url='https://github.com/sangheumcho/ksif',
    # download_url='https://github.com/sangheumcho/ksif/archive/master.zip',
    install_requires=[
        # 'h5py==2.8.0',
        'pandas',
        'numpy',
        'matplotlib',
        'scikit-learn',
        'scipy',
        'statsmodels',
        'patsy',
        'performanceanalytics',
        'tqdm',
        'requests',
        'tables',
        'python-dateutil',
        'urllib3',
        'xlrd',
        # 'dropbox'
    ],
    packages=find_packages(exclude=['tests*']),
    keywords=['ksif', 'portfolio', 'backtest', 'finance'],
    python_requires='>=3.5',
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 9. 14.
"""
from functools import wraps
from pandas import DataFrame, Series

from ..errors import EmptyResultException


def not_empty(func):
    """
    Check the result of this function is empty or not.
    If the result is empty, raise a runtime error.
    """

    @wraps(func)
    def check_not_empty(*args, **kwargs):
        result = func(*args, **kwargs)
        if (type(result) is DataFrame or type(result) is Series) and result.empty:
            raise EmptyResultException()

        return result

    return check_not_empty



# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 7. 18.
"""
from functools import wraps
from copy import deepcopy as dc


def memoize(copy=False):
    def deco_memoize(func):
        cache = func.cache = {}

        @wraps(func)
        def memoized_func(*args, **kwargs):
            key = str(args) + str(kwargs)
            if key not in cache:
                cache[key] = func(*args, **kwargs)
            if copy:
                return dc(cache[key])
            return cache[key]

        return memoized_func

    return deco_memoize



# -*- coding: utf-8 -*-
"""
:Author: Jaekyung Kim, Sangheum Cho
:Date: 2019. 7. 29.
"""
import itertools

# Base information
DATADATE = 'datadate'
CODE = 'code'
NAME = 'name'
DATE = 'date'
RET_1 = 'return_1'  # 1개월 수익
RET_3 = 'return_3'  # 3개월 수익
RET_6 = 'return_6'  # 6개월 수익
RET_12 = 'return_12'  # 12개월 수익
ADJP = 'adjp'  # 수정종가
ENDP = 'endp'  # 종가
MKTCAP = 'mktcap'  # 시가총액
BASE_INFORMATION = [CODE, NAME, DATE, RET_1, RET_3, RET_6, RET_12, ADJP, ENDP, MKTCAP]

# Fundamental
REVTQ = 'revtq'
REVT12 = 'revt12'
GPQ = 'gpq'
GP12 = 'gp12'
OPQ = 'opq'
OP12 = 'op12'
NIQ = 'niq'
NI12 = 'ni12'
ATQ = 'atq'
ATQ_MEAN4 = 'atq_mean4'
SEQ = 'seq'
SEQ_MEAN4 = 'seq_mean4'
LTQ = 'ltq'

CF_DEP = 'cf_dep'
CFOQ = 'cfoq'
CFO12 = 'cfo12'

A_TANG = 'a_tang'
A_TANG_MEAN12 = 'a_tang_mean12'
A_CUR = 'a_cur'
A_CUR_MEAN12 = 'a_cur_mean12'
L_CUR = 'l_cur'
L_CUR_MEAN12 = 'l_cur_mean12'


# 수익성지표
S_A = 's_a'  # 매출/자산 Sale over Assets
GP_A = 'gp_a'  # 매출총이익/자산 Gross Profit over Assets
OP_A = 'op_a'  # 영업이익/자산 EBIT over Assets
CF_A = 'cf_a'  # 영업활동으로 인한 현금흐름/자산 Cash Flow from operation over Assets
ROA = 'roa'  # 연간 순이익/자산 Return On Assets
ROE = 'roe'  # 연간 순이익/자본 Return On Equity
QROA = 'qroa'  # 분기간 순이익/자산
QROE = 'qroe'  # 분기간 순이익/자본
EBT_E = 'ebt_e'  # (순이익+세금)/자본
ROIC = 'roic'  # 투하자본수익률 Return On Invested Capital
GP_S = 'gp_s'  # 매출총이익/매출
DIVP = 'divp'  # 배당률 Dividend rate(12 month rolling)
PROFIT_FACTORS = [S_A, GP_A, OP_A, CF_A, ROA, ROE, QROA, QROE, EBT_E, ROIC, GP_S, DIVP]




# Filter
EXCHANGE = 'exchange'  # 거래소
FN_GUIDE_SECTOR = 'fn_guide_sector'  # FN Guide 섹트
FN_GUIDE_INDUSTRY_GROUP_27 = 'fn_guide_industry_group_27'  # FN Guide 산업구분
KRX_SECTOR = 'krx_sector'  # 한국거래소 섹터
HOLDING = 'holding'  # 지주사 여부
IS_MANAGED = 'is_managed'  # 관리종목 여부
IS_SUSPENDED = 'is_suspended'  # 거래정지 여부
FILTERS = [EXCHANGE, FN_GUIDE_SECTOR, FN_GUIDE_INDUSTRY_GROUP_27, KRX_SECTOR, HOLDING, IS_MANAGED,
           IS_SUSPENDED]

# Value factors
PER = 'per'  # 주가수익률 Per Earnings Ratio
PBR = 'pbr'  # 주가순자산비율 Price to Book-value Ratio
PSR = 'psr'  # 주가매출비율 Price to Sales Ratio
PCR = 'pcr'  # 주가현금흐름비율 Price to Cash flow Ratio
PGPR = 'pgpr'  # 주가매출총이익비율 Price to Gross Profit Ratio
POPR = 'popr'  # 주가영업이익비율 Price to Operating Profit Ratio
EV_EBITDA = 'ev_ebitda'  # 시장가격/세금이자지급전이익
EBIT_EV = 'ebit_ev'  # 영업이익/시장가격
CF_EV = 'cf_ev'  # 영업현금흐름/시장가격
S_EV = 's_ev'  # 매출/시장가격
E_P = 'e_p'  # 1/PER
B_P = 'b_p'  # 1/PBR
C_P = 'c_p'  # 1/PCR
S_P = 's_p'  # 1/PSR
GP_P = 'gp_p'  # 1/PGPR
OP_P = 'op_p'  # 1/POPR
VALUE_FACTORS = [PER, PBR, PSR, PCR, PGPR, POPR, EV_EBITDA, EBIT_EV, CF_EV, S_EV, E_P, B_P, C_P, S_P, GP_P, OP_P]

# market
PRCC = 'prcc'
RET = 'ret'

# Growth factors
SALESQOQ = 'salesqoq'  # 분기간 매출 변화율
GPQOQ = 'gpqoq'  # 분기간 매출총이익 변화율
OPQOQ = 'opqoq'  # 분기간 영업이익 변화율
ROAQOQ = 'roaqoq'  # 분기간 ROA 변화율
ROAYOY = 'roayoy'  # 연간 ROA 변화율
GP_SYOY = 'gp_syoy'  # 연간 매출총이익/매출 변화율
GP_AYOY = 'gp_ayoy'  # 연간 매출총이익/자산 변화율
ASSETSYOY = 'assetsyoy'  # 연간 자산 변화율
ASSETSQOQ = 'assetsqoq'  # 분기간 자산 변화율
GROWTH_FACTORS = [SALESQOQ, GPQOQ, OPQOQ, ROAQOQ, ROAYOY, GP_SYOY, GP_AYOY, ASSETSYOY, ASSETSQOQ]

# Momentum factors
MOM12_1 = 'mom12-1'  # 12개월 모멘텀 - 1개월 모멘텀
MOM6_1 = 'mom6-1'  # 6개월 모멘텀 - 1개월 모멘텀
MOM12 = 'mom12'  # 12개월 모멘텀
MOM6 = 'mom6'  # 6개월 모멘텀
MOM3 = 'mom3'  # 3개월 모멘텀
MOM1 = 'mom1'  # 1개월 모멘텀
MOMENTUM_FACTORS = [MOM12_1, MOM6_1, MOM12, MOM6, MOM3, MOM1]

# Safety factors
BETA_1D = 'beta_1d'  # 일일 베타
BETA_3M = 'beta_3m'  # 3개월 베타
BETA_5M = 'beta_5m'  # 5개월 베타
BETA_1W = 'beta_1w'  # 1주 베타
BETA_2W = 'beta_2w'  # 2주 베타
VOL_1D = 'vol_1d'  # 일일 변동성
VOL_3M = 'vol_3m'  # 3개월 변동성
VOL_5M = 'vol_5m'  # 5개월 변동성
VOL_1W = 'vol_1w'  # 1주 변동성
VOL_2W = 'vol_2w'  # 2주 변동성
LIQ_RATIO = 'liq_ratio'  # 유동비율
DEBT_RATIO = 'debt_ratio'  # 부채비율
EQUITY_RATIO = 'equity_ratio'  # 자본비율
SAFETY_FACTORS = [BETA_1D, BETA_1W, BETA_2W, BETA_3M, BETA_5M, VOL_1D, VOL_1W, VOL_2W, VOL_3M, VOL_5M, LIQ_RATIO,
                  DEBT_RATIO, EQUITY_RATIO]

# Liquidity factors
TRADING_VOLUME_RATIO = 'trading_volume_ratio'  # 거래비율
NET_PERSONAL_PURCHASE_RATIO = 'net_personal_purchase_ratio'  # 개인순매수비율
NET_INSTITUTIONAL_FOREIGN_PURCHASE_RATIO = 'net_institutional_foreign_purchase_ratio'  # 기관/외국인순매수비율
NET_INSTITUTIONAL_PURCHASE_RATIO = 'net_institutional_purchase_ratio'  # 기관순매수비율
NET_FINANCIAL_INVESTMENT_PURCHASE_RATIO = 'net_financial_investment_purchase_ratio'  # 금융투자순매수비율
NET_INSURANCE_PURCHASE_RATIO = 'net_insurance_purchase_ratio'  # 보험순매수비율
NET_TRUST_PURCHASE_RATIO = 'net_trust_purchase_ratio'  # 투신순매수비율
NET_PRIVATE_FUND_PURCHASE_RATIO = 'net_private_fund_purchase_ratio'  # 사모펀드순매수비율
NET_BANK_PURCHASE_RATIO = 'net_bank_purchase_ratio'  # 은행순매수비율
NET_ETC_FINANCE_PURCHASE_RATIO = 'net_etc_finance_purchase_ratio'  # 기타금융순매수비율
NET_PENSION_PURCHASE_RATIO = 'net_pension_purchase_ratio'  # 연기금순매수비율
NET_NATIONAL_PURCHASE_RATIO = 'net_national_purchase_ratio'  # 국가순매수비율
NET_ETC_CORPORATION_PURCHASE_RATIO = 'net_etc_corporation_purchase_ratio'  # 기타법인순매수비율
NET_FOREIGN_PURCHASE_RATIO = 'net_foreign_purchase_ratio'  # 외국인순매수비율
NET_REGISTERED_FOREIGN_PURCHASE_RATIO = 'net_registered_foreign_purchase_ratio'  # 등록외국인순매수비율
NET_ETC_FOREIGN_PURCHASE_RATIO = 'net_etc_foreign_purchase_ratio'  # 기타외국인순매수비율
FOREIGN_OWNERSHIP_RATIO = 'foreign'  # 외국인보유비중
SHORT_SALE_VOLUME_RATIO = 'short_sale_volume_ratio'  # 공매도거래비율
SHORT_SALE_BALANCE_RATIO = 'short_sale_balance_ratio'  # 공매도잔고비율
SHORT_SALE_BALANCE_MOM = 'short_sale_balance_mom'  # 월간 공매도잔고변화율
SHARE_LENDING_VOLUME_RATIO = 'share_lending_volume_ratio'  # 대차거래비율
SHARE_LENDING_BALANCE_RATIO = 'share_lending_balance_ratio'  # 대차잔고비율
SHARE_LENDING_BALANCE_MOM = 'share_lending_balance_mom'  # 월간 공매도잔고변화율
LIQUIDITY_FACTORS = [
    TRADING_VOLUME_RATIO, NET_PERSONAL_PURCHASE_RATIO, NET_INSTITUTIONAL_FOREIGN_PURCHASE_RATIO,
    NET_INSTITUTIONAL_PURCHASE_RATIO, NET_FINANCIAL_INVESTMENT_PURCHASE_RATIO, NET_INSURANCE_PURCHASE_RATIO,
    NET_TRUST_PURCHASE_RATIO, NET_PRIVATE_FUND_PURCHASE_RATIO, NET_BANK_PURCHASE_RATIO,
    NET_ETC_FINANCE_PURCHASE_RATIO, NET_PENSION_PURCHASE_RATIO, NET_NATIONAL_PURCHASE_RATIO,
    NET_ETC_CORPORATION_PURCHASE_RATIO, NET_FOREIGN_PURCHASE_RATIO, NET_REGISTERED_FOREIGN_PURCHASE_RATIO,
    NET_ETC_FOREIGN_PURCHASE_RATIO, FOREIGN_OWNERSHIP_RATIO, SHORT_SALE_VOLUME_RATIO, SHORT_SALE_BALANCE_RATIO,
    SHORT_SALE_BALANCE_MOM, SHARE_LENDING_VOLUME_RATIO, SHARE_LENDING_BALANCE_RATIO, SHARE_LENDING_BALANCE_MOM,
]

COMPANY_FACTORS = list(
    itertools.chain(
        # Values factors
        VALUE_FACTORS,
        # Profit factors
        PROFIT_FACTORS,
        # Growth factors
        GROWTH_FACTORS,
        # Momentum factors
        MOMENTUM_FACTORS,
        # Safety factors
        SAFETY_FACTORS,
        # Liquidity factors
        LIQUIDITY_FACTORS,
    )
)

# Macro factors
TERM_SPREAD_KOR = "term_spread_kor"  # 장단기 국채 금리차이 10 - 1, 한국 국채; 장기 경제 전망
TERM_SPREAD_US = "term_spread_us"  # 장단기 국채 금리차이 10 - 1, 미국 국채
CREDIT_SPREAD_KOR = "credit_spread_kor"  # 회사채 BBB- - AA- ; 신용위험도
LOG_USD2KRW = "log_usd2krw"  # 1달러 당 원화 가격의 로그
LOG_CHY2KRW = "log_chy2krw"  # 1위안 당 원화 가격의 로그
LOG_EURO2KRW = "log_euro2krw"  # 1유로 당 원화 가격의 로그
TED_SPREAD = "ted_spread"  # 리보금리 - 미국 국채 스프레드 (1개월물) ; 미국 역외 달러 조달의 위험성 ; 무역과 internal banking system의 안정성 지표
LOG_NYSE = "log_nyse"  # 뉴욕증권거래소 지수의 로그
LOG_NASDAQ = "log_nasdaq"  # 나스닥 거래소 지수의 로그
LOG_SEMI_CONDUCTOR = "log_semi_conductor"  # 반도체가격의 로그
LOG_DOLLAR_INDEX = "log_dollar_index"  # 달러 인덱스의 로그; 달러가 다른 통화대비 얼마나 강세인지 보여줌
LOG_OIL = "log_oil"  # 유가의 로그
LOG_EXPORT = "log_export"  # 한국의 수출량 로그
LOG_IMPORT = "log_import"  # 한국의 수입량 로그
LOG_INDUSTRY_PRODUCTION_US = "log_industry_production_us"  # 미국의 산업생산 로그
LOG_INDUSTRY_PRODUCTION_EURO = "log_industry_production_euro"  # 유로지역의 산업생산 로그
LOG_INDUSTRY_PRODUCTION_KOR = "log_industry_production_kor"  # 한국의 산업생산 로그

MACRO_FACTORS = [
    TERM_SPREAD_KOR, TERM_SPREAD_US, CREDIT_SPREAD_KOR, LOG_USD2KRW, LOG_CHY2KRW, LOG_EURO2KRW, TED_SPREAD, LOG_NYSE,
    LOG_NASDAQ, LOG_SEMI_CONDUCTOR, LOG_DOLLAR_INDEX, LOG_OIL,
]

# Benchmarks
MKF_2000 = 'MKF2000'
KOSDAQ = '코스닥'
KOSDAQ_LARGE = '코스닥 대형주'
KOSDAQ_MIDDLE = '코스닥 중형주'
KOSDAQ_SMALL = '코스닥 소형주'
KOSPI = '코스피'
KOSPI_200 = '코스피 200'
KOSPI_LARGE = '코스피 대형주'
KOSPI_MIDDLE = '코스피 중형주'
KOSPI_SMALL = '코스피 소형주'
TOTAL_LARGE = '코스피/코스닥 대형주 평균'
TOTAL_MIDDLE = '코스피/코스닥 중형주 평균'
TOTAL_SMALL = '코스피/코스닥 소형주 평균'
CD91 = 'CD 91일'

BENCHMARK_RET_1 = 'benchmark_return_1'
BENCHMARK_RET_3 = 'benchmark_return_3'
BENCHMARK_RET_6 = 'benchmark_return_6'
BENCHMARK_RET_12 = 'benchmark_return_12'

BENCHMARKS = [
    MKF_2000,
    KOSDAQ, KOSDAQ_LARGE, KOSDAQ_MIDDLE, KOSDAQ_SMALL,
    KOSPI, KOSPI_200, KOSPI_LARGE, KOSPI_MIDDLE, KOSPI_SMALL,
    TOTAL_LARGE, TOTAL_MIDDLE, TOTAL_SMALL,
    CD91,
]

# Factors
SMB = 'SMB'
HML = 'HML'


# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 8. 24.
"""
from unittest import TestCase

from ksif import *


class TestPeriodicStandardize(TestCase):

    def test_periodic_standardize(self):
        pf = Portfolio()

        prefix = 'std_'
        std_per = prefix + PER

        pf = pf.periodic_standardize(factor=PER, prefix=prefix)

        # Periodic means are almost 0.
        pf.groupby(by=DATE)[std_per].mean().apply(lambda x: self.assertAlmostEqual(x, 0))
